#!/bin/bash

#-------------------------------------------------------------
# SCRIPT PARA RESET DE FÁBRICA - FACTORY RESET
#
# - killall plcd, start e codesyscontrol
# - atualiza os arquivos padrões das redes: eths
# - atualiza o arquivo de configuração do Codesys
# - remove o conteúdo da pasta var/lib/weg/codesys
# - escreve no log que foi realizado o factory reset
# - reinicia o PLC
#
# script localizado em: opt/weg/webpage/bin/scripts/factoryReset.sh 
#
#-------------------------------------------------------------

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR


#-------------------------------------------------------
# Kill processes
#-------------------------------------------------------
function kill_plcd {
    until ! pgrep plcd; do sleep 2; pkill plcd; ((count++)); [ $count -eq 10 ] && break; done || true
}

function kill_start {
    until ! pgrep start; do sleep 2; pkill start; ((count++)); [ $count -eq 10 ] && break; done || true
}

function kill_codesyscontrol {
    until ! pgrep codesyscontrol; do sleep 2; pkill codesyscontrol; ((count++)); [ $count -eq 10 ] && break; done || true
}

#-------------------------------------------------------------
# Excluindo redes antigas e atribuindo as padrões
#-------------------------------------------------------------
function restore_networks {
    cd /etc/network/interfaces.d/ && rm eth*
    cp /opt/weg/backups/eth1 /etc/network/interfaces.d/
    cp /opt/weg/backups/eth2 /etc/network/interfaces.d/
    if [[ -f "/etc/network/interfaces.d/br1" ]]; then rm "/etc/network/interfaces.d/br1"; fi
}

#-------------------------------------------------------------
# Excluindo arquivo de configuração do Codesys e colando padrão
#-------------------------------------------------------------
function restore_cfg {
    rm /etc/weg/codesys/CODESYSControl.cfg
    cp /opt/weg/backups/codesys/CODESYSControl_factory.cfg /etc/weg/codesys/CODESYSControl.cfg
}

#-------------------------------------------------------------
# Excluindo arquivo de .cfg Shell do Codesys e colando padrão
#-------------------------------------------------------------
function restore_shellcfg {
    if [[ -f "/etc/weg/codesys/CODESYSControlShell.cfg" ]]; then rm /etc/weg/codesys/CODESYSControlShell.cfg; fi
    if [[ -f "/opt/weg/backups/codesys/CODESYSControl_shell.cfg" ]]; then cp /opt/weg/backups/codesys/CODESYSControl_shell.cfg /etc/weg/codesys/CODESYSControlShell.cfg; fi
}

#-------------------------------------------------------------
# Excluindo arquivo de configuração do wifi
#-------------------------------------------------------------
function restore_wifi {
    if [[ -f "/etc/weg/plcconfig.toml" ]]; then rm /etc/weg/plcconfig.toml; fi
    if [[ -f "/opt/weg/backups/plcconfig.toml" ]]; then cp /opt/weg/backups/plcconfig.toml /etc/weg/; fi
}

#-------------------------------------------------------
# Remove o conteúdo da pasta /var/lib/weg/codesys
#-------------------------------------------------------
function erase_codesysfolder {
    cd /var/lib/weg/codesys && rm -r * || true; rm -r .* || true
}

#-------------------------------------------------------
# Remove o conteúdo da pasta /opt/weg/webpage/bin/login
#-------------------------------------------------------
function erase_webpagecredentials {
    if [[ -f "/opt/weg/webpage/bin/login/credentials.db" ]]; then cd /opt/weg/webpage/bin/login/ && rm -r *; fi
}

#-------------------------------------------------------------
# Kill dhclient process
#-------------------------------------------------------------
function kill_dhclient {
    if grep -qi "dhcp" /etc/network/interfaces.d/eth1 || grep -qi "dhcp" /etc/network/interfaces.d/eth2; 
    then
        systemctl stop --force networking
        pkill -SIGKILL -f 'dhclient'
        pkill -SIGKILL -f 'dhclient'
    fi
}

#-------------------------------------------------------------
# Kill CodeMeter services
#-------------------------------------------------------------
function kill_codemeter {
    if pgrep -f 'CodeMeterLin'; then pkill -SIGKILL -f 'CodeMeterLin'; fi
    if pgrep -f 'CmWebAdmin';   then pkill -SIGKILL -f 'CmWebAdmin';   fi
}

#-------------------------------------------------------------
# Se for PLC500ED, atualiza config.toml
#-------------------------------------------------------------
function restore_toml {
    if [[ -f "/etc/core-app/config.toml" ]]; 
    then
        systemctl stop weg-coreapp
        
        rm /etc/core-app/config.toml

        if ls /var/lib/losant-edge-agent/data/*.db 1> /dev/null 2>&1; then
            rm -r /var/lib/losant-edge-agent/data/*.db
        fi
        
        if ls /var/lib/mosquitto/*.db 1> /dev/null 2>&1; then
            rm -r /var/lib/mosquitto/*.db
        fi

        if [[ -f "/opt/weg/backups/plc500ed/config.toml" ]]; then cp /opt/weg/backups/plc500ed/config.toml /etc/core-app/; fi

        systemctl restart weg-coreapp &

        systemctl restart logrotate
        
    fi
}

#-------------------------------------------------------------
# Remove logs com mais de 20 MB
#-------------------------------------------------------------
function remove_biglogs {
    find /var/log -maxdepth 1 -type f -size +20M -exec rm {} \;
}

#-------------------------------------------------------
# Reinicia o produto
#-------------------------------------------------------
function do_reboot {
    if [[ -d "/opt/core-app" ]]; then (systemctl stop weg-coreapp && sleep 1 && reboot) &
    else (sleep 1 && reboot) &
    fi
}

#-------------------------------------------------------
# Reinicia o produto
#-------------------------------------------------------
function set_approved {
    /opt/weg/scripts/plcleds.sh --testapproved
}

case "$1" in
        "-h" | "--help")
            echo "" &&
            echo "------------------------------------------------------" &&
            echo "plcfacReset HELP" &&
            echo "------------------------------------------------------" &&
            echo "[empty]       : Realiza o factoryReset, com o reboot no final." &&
            echo "--approved    : Realiza o factoryReset, setando os leds no final sem o reboot." &&
            echo "------------------------------------------------------" 
            ;;

        "" | "--factory")

            kill_plcd
            kill_start
            kill_codesyscontrol
            restore_networks
            restore_cfg
            restore_shellcfg
            restore_wifi
            erase_codesysfolder
            erase_webpagecredentials
            kill_dhclient
            kill_codemeter
            restore_toml
            remove_biglogs
            plcLog WARN script plcfacReset executed
            sync
            do_reboot

            ;;

        "--approved")

            kill_start
            kill_codesyscontrol
            restore_networks
            restore_cfg
            restore_shellcfg
            restore_wifi
            erase_codesysfolder
            erase_webpagecredentials
            kill_dhclient
            kill_codemeter
            restore_toml
            remove_biglogs
            plcLog TEST Approved GIGA tests
            sync
            set_approved

            ;;

        *)
            echo "No option provided. Try -h to see the help."
            ;;

esac


