#!/bin/bash

#-------------------------------------------------------------
# SCRIPT PARA A CONFIGURACAO DURANTE O PRIMEIRO BOOT 
# 
# Junho/2023
# 
# Os leds da SPI alternam entre verde e vermelho até a 
# finalização do script.
#
# - Seta pino de enable (DOs e SPI)
# - Seta a data e hora para o dia 2023-01-01
# - Printa no log que o script iniciou
# - Desabilita serviço do Bluetooth 
# - Pisca leds de gravação assim que o serviço da SPI estiver disponível
# - Cria a chave SSH e reinicia o servico SSH
# - Configuração das variáveis de ambiente do uboot e configura o M4
# - Printa finalização do script finalizou
# - Troca o console do kernel para a UART7
# - Unmask e enable plchwconfig
# - Desliga padrão de leds
# - Liga os leds da SPI em verde até que o dispositivo seja reiniciado
# - Executa plcd
# - Desabilita o serviço plcfirstboot
# - sync
#
#-------------------------------------------------------------


#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR


#-------------------------------------------------------------
# Debug echo Function
#-------------------------------------------------------------
function debug_echo {
    if [[ "$DEBUG_FLAG" -ne 0 && -n "$DEBUG_FLAG" ]]; then
        echo "DEBUG_PLCINIT: $1" | tee /dev/kmsg;   
    fi
}

#-------------------------------------------------------------
# Try then test Function
#-------------------------------------------------------------
function try_then_execute {
    local test="$1"
    local command="$2"
    local tries="$3"
    local count=0

    debug_echo "entered try_then_execute: $test"

    if eval "$test"; then

        if [[ -z "$tries" ]]; then tries=10; fi

        while [[ "$count" -lt "$tries" ]]; do
            eval "$command" || true; 
            if ! eval "$test"; then count=100; debug_echo "rodou de primeira"; return 0; fi
            ((count++))
            sleep 1
            debug_echo "trying to execute the command again..."
            if [[ "$count" -ge "$tries" ]]; then
                return 1
            fi
        done
        return 0
    else
        debug_echo "false condition"
        return 0
    fi 
}


#-------------------------------------------------------------
# Carregar drive USB/Ethernet  na inicialização
#-------------------------------------------------------------
function config_USB2 {

    try_then_execute "! lsmod | grep -q g_ether" "modprobe g_ether" 
    try_then_execute "! ip link show usb0" "sleep 1"
    try_then_execute "! ifconfig usb0 | grep -q '192.168.234.234'" "ifconfig usb0 192.168.234.234 up" 
    try_then_execute "! ip link show usb2" "ip link set usb0 down && ip link set usb0 name usb2"
    try_then_execute "! ifconfig usb2 | grep -q 'UP'" "ip link set usb2 up" 
}


#-------------------------------------------------------------
# Cleaning
#-------------------------------------------------------------
function cleaningfiles {
    if [[ -e "/var/log/weg/plcLogs.log" ]];     then echo > /var/log/weg/plcLogs.log;     fi
    if [[ -e "/var/log/weg/plcxWegLib.log" ]];  then echo > /var/log/weg/plcxWegLib.log;  fi
    if [[ -e "/var/log/weg/plcFirmware.log" ]]; then echo > /var/log/weg/plcFirmware.log; fi
}


#-------------------------------------------------------------
# Hardware version (create plcgiga.lock if it's the flashing giga)
#-------------------------------------------------------------
function plcgigalock {
    
    [[ ! -d "/sys/class/gpio/gpio44" ]] && echo 44 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio44" ]]   && echo in > /sys/class/gpio/gpio44/direction

    [[ ! -d "/sys/class/gpio/gpio45" ]] && echo 45 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio45" ]]   && echo in > /sys/class/gpio/gpio45/direction

    [[ ! -d "/sys/class/gpio/gpio46" ]] && echo 46 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio46" ]]   && echo in > /sys/class/gpio/gpio46/direction

    GPIO1_PATH="/sys/class/gpio/gpio44/value"
    GPIO2_PATH="/sys/class/gpio/gpio45/value"
    GPIO3_PATH="/sys/class/gpio/gpio46/value"

    HW1_VAL=$(cat "$GPIO1_PATH"); 
    HW2_VAL=$(cat "$GPIO2_PATH"); 
    HW3_VAL=$(cat "$GPIO3_PATH"); 

    HW_VERSION="$HW3_VAL$HW2_VAL$HW1_VAL"

    if [[ "$HW_VERSION" == "010" ]]; then 
        touch $GIGALOCKFILE; 
        if [[ ! -f $GIGALOCKFILE ]]; then sleep 2; touch $GIGALOCKFILE; plcLog WARN There was no plcgiga.lock; fi
        if [[ ! -f $GIGALOCKFILE ]]; then sleep 2; touch $GIGALOCKFILE; plcLog ERROR There was no plcgiga.lock; fi
    fi
}


#-------------------------------------------------------------
# Configurando e setando GPIO66 (PINO 7 - Enable 4 ultimas saidas e LEDSs SPI)
#-------------------------------------------------------------
function enableleds {
    [[ ! -d "/sys/class/gpio/gpio66" ]] && echo 66 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio66" ]]   && echo out > /sys/class/gpio/gpio66/direction
    [[ -d "/sys/class/gpio/gpio66" ]]   && echo 1 > /sys/class/gpio/gpio66/value
}


#-------------------------------------------------------------
# Get firmware version
#-------------------------------------------------------------
function getfirmversion {
    firmversion=$(dpkg -s firmware-plc500 | grep '^Version:' | sed 's/Version: //');
}


#-------------------------------------------------------
# Ajuste da data e hora do hardware e do sistema
#-------------------------------------------------------
function setclock {
    hwclock --set --date "01/01/2024 00:00:00"
    hwclock --hctosys
}


#-------------------------------------------------------------
# Log com a informação da versão de FW do firstboot
#-------------------------------------------------------------
function setlog {
    touch /var/log/weg/plcFirmware.log;
    echo "$(date +'%Y-%m-%d %T') firstboot $firmversion" >> /var/log/weg/plcFirmware.log
    plcLog FIRST "$firmversion" 
    sync
}


#-------------------------------------------------------
# Executa o plcleds.sh --grav
#-------------------------------------------------------
function ledsgrav {
    /opt/weg/scripts/plcleds.sh --grav &
}


#-------------------------------------------------------
# Cria a chave SSH e reinicia o serviço
#-------------------------------------------------------
function configssh {
    dpkg-reconfigure openssh-server
    systemctl restart ssh
}


#-------------------------------------------------------
# Configuração das variáveis de ambiente do uboot
#-------------------------------------------------------
function ubootvars {
    fw_setenv --script /etc/weg/uboot-vars
    rm /etc/weg/uboot-vars
}


#-------------------------------------------------------
# Troca o console do kernel da UART1 para a UART7
#-------------------------------------------------------
function changeconsole {
    fw_setenv console ttymxc6
}


#-------------------------------------------------------
# Desmascara e habilita o serviço do plchwconfig
#-------------------------------------------------------
function unmaskplchwconfig {
    systemctl unmask plchwconfig.service
    systemctl enable plchwconfig.service
    systemctl daemon-reload
    sync
}


#-------------------------------------------------------------
# Executa o daemon plcd
#-------------------------------------------------------------
function execplcd {
    if [[ -e "/usr/bin/plcd" ]]; 
    then 
        chmod +x /usr/bin/plcd; plcd &
        
        pgrep -x plcd > /dev/null; plcdrunning=$?; 
        if [[ "$plcdrunning" != "0" ]]; 
        then 
            /usr/bin/plcd &
        fi

    else 
        plcLog ERROR there was no plctools inside /usr/bin
    fi
}


#-------------------------------------------------------------
# Disable Bluetooth
#-------------------------------------------------------------
function disableble {
    systemctl disable bluetooth &
}


#-------------------------------------------------------
# Executa plcleds.sh --final e aguarda reboot
#-------------------------------------------------------
function execplcledsfinal {
    count=0; until ! pgrep plcleds.sh; do sleep 1; pkill plcleds.sh; ((count++)); [ $count -eq 10 ] && plcLog ERROR plcleds could not be killed && break; done || true
    sleep 3; 
    /opt/weg/scripts/plcleds.sh --final &
}



#-------------------------------------------------------------
# Variables
#-------------------------------------------------------------
GIGALOCKFILE="/var/lib/weg/flags/plcgiga.lock"


#-------------------------------------------------------
# Main
#-------------------------------------------------------
cleaningfiles
config_USB2
getfirmversion
plcgigalock
enableleds
setclock
setlog
disableble
ledsgrav
configssh
ubootvars
changeconsole
unmaskplchwconfig
execplcd

plcLog INFO script firstboot finalizado
systemctl disable plcfirstboot

execplcledsfinal
sync