#!/bin/bash

#-------------------------------------------------------------
# CONFIGURACAO INICIAL DOS PLCs
#
# September/2023
# 
#-------------------------------------------------------------

#-------------------------------------------------------------
# Carrega as variáveis e funções utilizadas pelos PLCs WEG
#-------------------------------------------------------------
source /opt/weg/scripts/plcDefines.sh


#-------------------------------------------------------------
# Trap to catch line errors
#-------------------------------------------------------------
trap 'plcLog ERROR Trap!' ERR


#-------------------------------------------------------------
# Debug echo Function
#-------------------------------------------------------------
function debug_echo {
    if [[ -n "$DEBUG_FLAG" ]]; then  # Exibe a mensagem apenas se DEBUG_FLAG estiver definida
        echo "DEBUG_PLCINIT: $1" | tee /dev/kmsg;   
    fi
}


#-------------------------------------------------------
# Security Mode
#-------------------------------------------------------
function security_mode {

    debug_echo "entered securitymode"

    if [[ -e "$SECURITYLOCKFILE" ]]; then
        plcLog WARN protection mode has been enabled
        try_then_execute "test -f $SECURITYLOCKFILE" "rm -f $SECURITYLOCKFILE" || plcLog ERROR securitylockfile could not be removed
        try_then_execute "pgrep plcleds.sh" "pkill plcleds.sh" || plcLog ERROR plcleds could not be killed
        /opt/weg/scripts/plcleds.sh --develop &

        create_hash_and_pass
        execute_plcd
        rename_ETHs
        config_USB2

        # Verifica se existe o arquivo de configuração br1
        if [[ -f "/etc/network/interfaces.d/br1" ]]; 
        then
            ifconfig br1 192.168.1.10 255.255.255.0
            if [[ -f "/etc/network/interfaces.d/eth1" ]]; then ifconfig eth1 192.168.1.10 255.255.255.0; fi
            if [[ -f "/etc/network/interfaces.d/eth2" ]]; then ifconfig eth2 192.168.1.10 255.255.255.0; fi
        else
            if [[ -f "/etc/network/interfaces.d/eth1" ]]; then ifconfig eth1 192.168.1.10 255.255.255.0; fi
            if [[ -f "/etc/network/interfaces.d/eth2" ]]; then ifconfig eth2 192.168.2.10 255.255.255.0; fi
        fi

        systemctl start ssh

        exit 0;
    fi

    (plctools --hw_version && HW_VERSION=$?) || { HW_VERSION=$?; true; }

    if [[ "$HW_VERSION" == "100" ]] && [[ -e "$FILE0" ]];
    then
        try_then_execute "pgrep plcleds.sh" "pkill plcleds.sh" || plcLog ERROR plcleds could not be killed
        /opt/weg/scripts/plcleds.sh --invalid &
        exit 0
    fi
}

#-------------------------------------------------------------
# Verify if year < 2024
#-------------------------------------------------------------
function verify_date {

    debug_echo "entered verify_date"

    local current_year=$(date +"%Y")

    if [[ "$current_year" -lt 2024 ]]; then
        hwclock --set --date "01/01/2024 00:00:00"
        hwclock --hctosys
        plcLog WARN date was earlier than 2024!
        sync
    fi
}


#-------------------------------------------------------------
# Check if the plcInstall file exists
#-------------------------------------------------------------
function check_plcInstall {
    if [[ -f "/opt/weg/scripts/plcInstall.sh" ]]; 
    then 
        /opt/weg/scripts/plcInstall.sh && rm /opt/weg/scripts/plcInstall.sh
    fi
}

#-------------------------------------------------------------
# Check if the serial number file exists and create if not
#-------------------------------------------------------------
function verify_serialnumber {
    
    SERIALWEG=$(fw_printenv "serialweg" 2>/dev/null | awk -F'=' '{print $2}')
    
    if [[ ! -e "/etc/weg/serialweg.json" || -s "/etc/weg/serialweg.json" && -n "$SERIALWEG" ]]; then
         echo "$SERIALWEG" > "/etc/weg/serialweg.json";
    fi
}

#-------------------------------------------------------------
# Remove files not necessary
#-------------------------------------------------------------
function rm_files {
    if ls /var/log/weg/logPLC*.csv 1> /dev/null 2>&1; then rm /var/log/weg/logPLC*.csv; fi
    if ls /var/log/weg/core-app 1> /dev/null 2>&1; then rm -r /var/log/weg/core-app; fi
}


#-------------------------------------------------------------
# Gera a hash hexadecimal de 8 caracteres e o password (exceto PLC500ED)
#-------------------------------------------------------------
function create_hash_and_pass {
    if [[ "$HW_VERSION" != "3" ]];
    then

        # Test and save old hash
        if [[ -f "/etc/weg/plchash.json" ]]; 
        then old_hash=$(cat /etc/weg/plchash.json); 
        else touch "/etc/weg/plchash.json"; 
        fi

        # Create new hash and change password
        (openssl rand -hex 4 > /etc/weg/plchash.json && /opt/weg/scripts/plcssh) &

    fi
}


#-------------------------------------------------------------
# Lê o pino de boot safe
#-------------------------------------------------------------
function read_bootsafe {
    [[ ! -d "/sys/class/gpio/gpio76" ]] && echo 76 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio76" ]]   && echo in > /sys/class/gpio/gpio76/direction
    [[ -d "/sys/class/gpio/gpio76" ]]   && bootsafepin=$(cat /sys/class/gpio/gpio76/value)
    [[ -d "/sys/class/gpio/gpio76" ]]   && echo 76 > /sys/class/gpio/unexport
}


#-------------------------------------------------------------
# Executa o daemon plcd
#-------------------------------------------------------------
function execute_plcd {
    if [[ -f "/usr/bin/plcd" ]]; then chmod +x /usr/bin/plcd && plcd & fi
}


#-------------------------------------------------------
# Seta a temperatura passiva (reduz clock) para 90 graus
#-------------------------------------------------------
function set_temperature {
    debug_echo "entered set_temperature"
    echo 90000 > /sys/class/thermal/thermal_zone0/trip_point_0_temp
}


#-------------------------------------------------------------
# Configurando e setando GPIO103 (PINO 187 - USB Power)
#-------------------------------------------------------------
function config_USB1 {
    [[ ! -d "/sys/class/gpio/gpio103" ]] && echo 103 > /sys/class/gpio/export
    [[ -d "/sys/class/gpio/gpio103" ]]   && echo out > /sys/class/gpio/gpio103/direction
    [[ -d "/sys/class/gpio/gpio103" ]]   && echo 1 > /sys/class/gpio/gpio103/value
}


#-------------------------------------------------------------
# Try then test Function
#-------------------------------------------------------------
function try_then_execute {
    local test="$1"
    local command="$2"
    local tries="$3"
    local count=0

    debug_echo "entered try_then_execute: $test"

    if eval "$test"; then

        if [[ -z "$tries" ]]; then tries=10; fi

        while [[ "$count" -lt "$tries" ]]; do
            eval "$command" || true; 
            if ! eval "$test"; then count=100; debug_echo "rodou de primeira"; return 0; fi
            ((count++))
            sleep 1
            debug_echo "trying to execute the command again..."
            if [[ "$count" -ge "$tries" ]]; then
                return 1
            fi
        done
        return 0
    else
        debug_echo "false condition"
        return 0
    fi 
}

#-------------------------------------------------------------
# Renomeia redes
#-------------------------------------------------------------
function rename_ETHs {
    debug_echo "entered rename_ETHs. HW_VERSION=$HW_VERSION"

        if [[ "$HW_VERSION" -ne "0" ]];  # Se HW não for o PLC410
        then 
            if ! ip link show eth2; then

                try_then_execute "! ip link show eth1" "sleep 1"
                try_then_execute "! ip link show eth2" "ip link set eth1 down && ip link set eth1 name eth2"
                try_then_execute "! ifconfig eth2 | grep -q 'UP'" "ip link set eth2 up"

                try_then_execute "! ip link show eth0" "sleep 1"
                try_then_execute "! ip link show eth1" "ip link set eth0 down && ip link set eth0 name eth1"
                try_then_execute "! ifconfig eth1 | grep -q 'UP'" "ip link set eth1 up" 
            fi
            
        else
            if ! ip link show eth1; then

                try_then_execute "! ip link show eth0" "sleep 1"
                try_then_execute "! ip link show eth1" "ip link set eth0 down && ip link set eth0 name eth1"
                try_then_execute "! ifconfig eth1 | grep -q 'UP'" "ip link set eth1 up" 
            fi
        fi
}

#-------------------------------------------------------------
# Configura redes
#-------------------------------------------------------------
function config_ETHs {

    debug_echo "entered config_eths. HW_VERSION=$HW_VERSION"

    if [[ "$bootsafepin" == "0" ]]
    then

        if [[ "$HW_VERSION" -ne "0" ]];  # Se HW não for o PLC410
        then 
            if ! ip link show eth2; then

                try_then_execute "! ip link show eth1" "sleep 1"
                try_then_execute "! ip link show eth2" "ip link set eth1 down && ip link set eth1 name eth2"
                try_then_execute "! ifconfig eth2 | grep -q 'UP'" "ip link set eth2 up"

                try_then_execute "! ip link show eth0" "sleep 1"
                try_then_execute "! ip link show eth1" "ip link set eth0 down && ip link set eth0 name eth1"
                try_then_execute "! ifconfig eth1 | grep -q 'UP'" "ip link set eth1 up" 
            fi
            
        else
            if ! ip link show eth1; then

                try_then_execute "! ip link show eth0" "sleep 1"
                try_then_execute "! ip link show eth1" "ip link set eth0 down && ip link set eth0 name eth1"
                try_then_execute "! ifconfig eth1 | grep -q 'UP'" "ip link set eth1 up" 
            fi
        fi

        # Verifica se existe o arquivo de configuração br1
        if [[ -f "/etc/network/interfaces.d/br1" ]]; 
        then
            BR1_MODE=$(grep -oP '(?<=inet )[^ ]*' /etc/network/interfaces.d/br1)

            if [[ "$BR1_MODE" == "dhcp" ]]; 
            then 
                /opt/weg/scripts/plcbridge.sh --add 192.168.1.10 255.255.255.0 

            elif [[ "$BR1_MODE" == "static" ]];
            then

                BR1_IP=$(grep -oP '(?<=address )[^ ]*' /etc/network/interfaces.d/br1)
                BR1_NET=$(grep -oP '(?<=netmask )[^ ]*' /etc/network/interfaces.d/br1)

                /opt/weg/scripts/plcbridge.sh --add "$BR1_IP" "$BR1_NET"
                iptables -A FORWARD -i br1 -o br1 -j ACCEPT
            fi

        # Não existe br1, configuração independente
        else

            #-------------------------------------------------------------
            # Atribui IPs (static), limpa e busca IPs (dhcp) e limita velocidade ETH1 (PLC410)
            #-------------------------------------------------------------
            if [[ -f "/etc/network/interfaces.d/eth1" ]];
            then 
                ETH1_MODE=$(grep -oP '(?<=inet )[^ ]*' /etc/network/interfaces.d/eth1)
            else
                cp /opt/weg/backups/eth1  /etc/network/interfaces.d/
                ETH1_MODE=$(grep -oP '(?<=inet )[^ ]*' /etc/network/interfaces.d/eth1)
                plcLog WARN There was not eth1 inside networks
            fi

            if [[ "$HW_VERSION" != "0" ]];
            then
                if [[ -f "/etc/network/interfaces.d/eth2" ]];
                then
                    ETH2_MODE=$(grep -oP '(?<=inet )[^ ]*' /etc/network/interfaces.d/eth2)
                else
                    cp /opt/weg/backups/eth2  /etc/network/interfaces.d/
                    ETH2_MODE=$(grep -oP '(?<=inet )[^ ]*' /etc/network/interfaces.d/eth2)
                    plcLog WARN There was not eth2 inside networks
                fi
            fi

            if [[ "$ETH1_MODE" == "dhcp" ]]; then ip addr flush dev eth1; fi
            if [[ "$ETH2_MODE" == "dhcp" ]]; then ip addr flush dev eth2; fi

            if [[ "$ETH1_MODE" == "static" ]];
            then
                ETH1_IP=$(grep -oP '(?<=address )[^ ]*' /etc/network/interfaces.d/eth1)
                ETH1_NET=$(grep -oP '(?<=netmask )[^ ]*' /etc/network/interfaces.d/eth1)
                
                if [[ -n "$ETH1_IP" ]] && [[ -n "$ETH1_NET" ]];
                then
                    ifconfig eth1 "$ETH1_IP" netmask "$ETH1_NET"
                else
                    plcLog WARN parameter empty. ETH1_IP: $ETH1_IP, ETH1_NET: $ETH1_NET
                fi
            fi

            # Apenas configura a ETH2 se não for o PLC410
            if [[ "$HW_VERSION" != "0" ]]; 
            then 
                if [[ "$ETH2_MODE" == "static" ]];
                then
                    ETH2_IP=$(grep -oP '(?<=address )[^ ]*' /etc/network/interfaces.d/eth2)
                    ETH2_NET=$(grep -oP '(?<=netmask )[^ ]*' /etc/network/interfaces.d/eth2)
                    
                    if [[ -n "$ETH2_IP" ]] && [[ -n "$ETH2_NET" ]];
                    then
                        ifconfig eth2 "$ETH2_IP" netmask "$ETH2_NET"
                    else
                    plcLog WARN parameter empty. ETH2_IP: $ETH2_IP, ETH2_NET: $ETH2_NET
                    fi
                fi
            fi
        fi

        # Limita para 100 Mb/s a ethernet do PLC410
        if [[ "$HW_VERSION" == "0" ]]; then ethtool -s eth1 advertise 0x00F; fi
        
    else
        plcLog ERROR bootsafepin is ON!

    fi
}


#-------------------------------------------------------------
# Carregar drive USB/Ethernet  na inicialização
#-------------------------------------------------------------
function config_USB2 {

    try_then_execute "! lsmod | grep -q g_ether" "modprobe g_ether" 
    try_then_execute "! ip link show usb0" "sleep 1"
    try_then_execute "! ifconfig usb0 | grep -q '192.168.234.234'" "ifconfig usb0 192.168.234.234 up" 
    try_then_execute "! ip link show usb2" "ip link set usb0 down && ip link set usb0 name usb2"
    try_then_execute "! ifconfig usb2 | grep -q 'UP'" "ip link set usb2 up" 
}


#-------------------------------------------------------
# Config filter for sync in Codesys V3 protocol
#-------------------------------------------------------
function set_V3filter {
    dpkg -l | grep iptables > /dev/null 2>&1
    if [[ $? -eq 0 ]]; then

        if ! lsmod | grep -q xt_recent; 
        then
            n2=0
            until [[ "$n2" -ge "10" ]];
            do
                if ! lsmod | grep -wq "xt_recent"; 
                then
                    modprobe xt_recent
                    sleep 1; n2=$((n2+1));
                else    
                    n2=100;
                fi       
            done
        fi

        if [[ "$n2" -lt "100" ]]; then
            plcLog ERROR iptables: modprobe xt_recent
        fi

        iptables-legacy -A INPUT -p tcp --dport 11740 --tcp-flags FIN FIN -m recent --name ignore_syn --set
        if [[ $? -ne 0 ]]; then
            plcLog ERROR Filter iptables-legacy FIN FIN
        fi

        iptables-legacy -A INPUT -p tcp --dport 11740 --tcp-flags SYN SYN -m recent --rcheck --name ignore_syn --seconds 2 -j DROP
        if [[ $? -ne 0 ]]; then
            plcLog ERROR Filter iptables-legacy SYN SYN
        fi

    else
        plcLog ERROR iptables not installed
    fi
}


#-------------------------------------------------------
# Verifica existência de /dev/intrabus
#-------------------------------------------------------
function verify_intrabus {
    if [[ ! -e "$FILE0" ]];
    then 
        try_then_execute "! lsmod | grep -wq intrabus" "modprobe intrabus" 
    fi
}


#-------------------------------------------------------
# Monta a pasta e inicia o FTP server
#-------------------------------------------------------
function config_FTP {

    if dpkg -l | grep -qw "vsftpd";
    then
        if [[ ! -f /etc/ssl/private/vsftpd.pem ]];
        then
            cd /opt/weg/scripts/ftp && ./firstconfigFTP.sh
            systemctl disable vsftpd;
            systemctl stop vsftpd;
            plcLog INFO FTP first configuration
        fi  
    fi

    ftp_login="wegftp"

    if [[ -f /etc/ssl/private/vsftpd.pem ]];
    then
        if [[ ! -d "/var/lib/weg/codesys/ftp" ]]; 
        then 

            #-------------------------------------------------------------
            # Cria a pasta codesys/ftp
            #-------------------------------------------------------------
            mkdir -p /var/lib/weg/codesys/ftp;
            
            #-------------------------------------------------------------
            # Cria um link entre as duas pastas
            #-------------------------------------------------------------
            mount --bind /var/lib/weg/codesys/ftp /home/$ftp_login/ftp/files;
            
            #-------------------------------------------------------------
            # Altera a permissão da pasta
            #-------------------------------------------------------------
            grep -q "^wegftp:" /etc/group || groupadd wegftp

            chown $ftp_login:$ftp_login /home/$ftp_login/ftp/files;                  
            
            chmod -R 755 /home/$ftp_login/ftp/files;
            
        else
            if [[ ! -d "/home/$ftp_login/ftp/files" ]]; 
            then
                mkdir -p /home/$ftp_login/ftp/files
            fi

            mount --bind /var/lib/weg/codesys/ftp /home/$ftp_login/ftp/files;
            
        fi
        
        /opt/weg/scripts/ftp/initFTP.sh;
        
    fi
}


#-------------------------------------------------------
# Executa o serviço da webpage se não for PLC500ED
#-------------------------------------------------------
function start_webpage {
    if [[ "$HW_VERSION" != "3" ]]; then systemctl start plcwebpage > /dev/null 2>&1 & fi
}

#-------------------------------------------------------
# Executa plcleds.sh --invalid em caso de erro
#-------------------------------------------------------
function leds_error {

    try_then_execute "pgrep plcleds.sh" "pkill plcleds.sh" || plcLog ERROR plcleds could not be killed

    /opt/weg/scripts/plcleds.sh --invalid &

    if [[ "$HW_VERSION" == "100" ]]; then 
        plcLog ERROR Firmware and hardware are different!
        plctools --log -difhwfw
    fi

    if [[ ! -e "$FILE0" ]]; then 
        plcLog ERROR $FILE0 not loaded
        plctools --log -intrabus
    fi
}


#-------------------------------------------------------
# Finishes --loading and executes --init
#-------------------------------------------------------
function leds_init {
    try_then_execute "test -f $LEDSLOCKFILE" "rm $LEDSLOCKFILE" || plcLog ERROR Could not remove plcleds.lock
}


#-------------------------------------------------------
# Executa os scripts para montagem USB e SD card 
#-------------------------------------------------------
function mount_devices {
    /opt/weg/scripts/plcaddusb.sh
    /opt/weg/scripts/plcaddsd.sh
}


#-------------------------------------------------------
# Start CodeMeter
#-------------------------------------------------------
function start_codemeter {
    systemctl start codemeter-webadmin
    systemctl start codemeter
}


#-------------------------------------------------------
# Verify Application exists
#-------------------------------------------------------
function verify_application {
    
    if [[ -f "/etc/weg/codesys/CODESYSControl.cfg" ]]; then
        APPLICATION=$(sed -n 's/^Application\.1=//p' "/etc/weg/codesys/CODESYSControl.cfg")

        if [[ -n "$APPLICATION" ]]; then 
        
            if ! [[ -d "/var/lib/weg/codesys/PlcLogic/$APPLICATION" ]]; then
                sleep 1; 
            fi
        fi
    fi
}

#-------------------------------------------------------
# Executa o Codesys através do start
#-------------------------------------------------------
function start_codesys {
    chmod 700 /opt/weg/codesys/start && /opt/weg/codesys/start >/dev/null 2>&1 & 
}


#-------------------------------------------------------
# Verifica o hardware, /dev e inicializa o Codesys
#-------------------------------------------------------
function verify_all {
    (plctools --hw_version && HW_VERSION=$?) || { HW_VERSION=$?; true; }

    if [[ "$HW_VERSION" != "100" ]] && [[ -e "$FILE0" ]];
    then
        mount_devices
        start_codemeter
        leds_init
        verify_application
        start_codesys
        touch $INITLOCKFILE         # Flag que sinaliza que o plcinit foi finalizado com êxito
    else
        leds_error
    fi
}


#-------------------------------------------------------------
# Variables
#-------------------------------------------------------------
#DEBUG_FLAG="comment_to_avoid_logs"
FLAGS_PATH="/tmp/flags"; mkdir -p "${FLAGS_PATH}"
LEDSLOCKFILE="${FLAGS_PATH}"/plcleds.lock
INITLOCKFILE="${FLAGS_PATH}"/plcinit.lock
FLASHFLAGS_PATH="/var/lib/weg/flags"; mkdir -p "${FLASHFLAGS_PATH}"
SECURITYLOCKFILE="${FLASHFLAGS_PATH}"/securitymode.lock
HW_VERSION=$(grep -oP '(?<="HW_VERSION": )[0-9]+' /etc/weg/firstBoot.json)
FILE0="/dev/intrabus"


#-------------------------------------------------------
# Main
#-------------------------------------------------------
security_mode
verify_date
check_plcInstall
/opt/weg/scripts/plcwifi.sh --wifi_coreapp > /dev/null 2>&1 &
rm_files
verify_serialnumber
create_hash_and_pass
read_bootsafe
execute_plcd
set_temperature
config_USB1
config_ETHs
config_USB2
set_V3filter
verify_intrabus
config_FTP
start_webpage
verify_all


#-------------------------------------------------------
# Log
#-------------------------------------------------------
plcLog INFO script plcinit executado "$old_hash"


#-------------------------------------------------------
# Sync all cached file data of the current user
#-------------------------------------------------------
sync
